@@ -7,6 +7,7 @@ from sklearn.model_selection import KFold, train_test_split
 
 import mondobrain as mb
 from mondobrain.core.frame import MondoDataFrame
+from mondobrain.core.meta import BaseExploration
 from mondobrain.core.series import MondoSeries
 from mondobrain.dd_transformer import DDTransformer
 from mondobrain.error import NotEnoughPointsError
@@ -69,31 +70,13 @@ class Solver:
     def _is_one_dim_solve(explorable_vars):
         return explorable_vars.shape[1] == 1
 
-    @staticmethod
-    def _get_time_out(
-        explorable_vars, lg_timeout_coefficient=3.5, max_outer_loop_timeout=90
-    ):
-        number_of_points = explorable_vars.shape[0]
-        number_of_active_vars = explorable_vars.shape[1]
-        min_timeout = np.minimum(
-            int(
-                lg_timeout_coefficient
-                * np.sqrt(number_of_points * number_of_active_vars / 2)
-            ),
-            max_outer_loop_timeout,
-        )
-
-        timeout = np.maximum(1, min_timeout)
-
-        return timeout
-
     @staticmethod
     def _get_timeout(n_points, n_features, lg_coefficient=3.5, max_cycles=90):
 
         lg_timeout = int(lg_coefficient * np.sqrt(n_points * n_features / 2))
         min_timeout = np.minimum(lg_timeout, max_cycles)
 
-        return np.maximum(1, min_timeout)
+        return int(np.maximum(1, min_timeout))
 
     @staticmethod
     def _get_dataset_buffer(df: pd.DataFrame):
@@ -343,7 +326,7 @@ class Solver:
 
         evaluate(mdf)
 
-    def fit_meta(self, m_X, m_y, exp):
+    def fit_meta(self, m_X, m_y, exp: BaseExploration):
         """
         Discover feature rules with the provided data.
 
@@ -356,7 +339,7 @@ class Solver:
         m_y: MondoSeries of shape (n_samples,)
             Where n_samples is the number of samples and aligns with n_samples of m_X
 
-        exp: a core.meta.exploration object that extends BaseExploration
+        exp: an object that extends core.meta.BaseExploration
             The type of exploration used to discover feature rules
 
         """
@@ -370,15 +353,7 @@ class Solver:
         encoder = DDTransformer(mdf)
         edf = utilities.encode_dataframe(mdf, encoder)
         outcome_encoded = encoder.original_to_encoded_column(outcome)
-        if (
-            mdf[outcome].var_type == "discrete"
-            and hasattr(exp, "classes")
-            and exp.classes is not None
-            and len(exp.classes) > 0
-        ):
-            exp.classes = [
-                utilities.encode_value(mdf, outcome, x, encoder) for x in exp.classes
-            ]
+        exp.encode_options(mdf, outcome, encoder)
 
         edf, _ = utilities.sample_if_needed(edf, None, outcome_encoded)
 
@@ -386,7 +361,6 @@ class Solver:
         timeout = self._get_timeout(
             edf.shape[0], edf.shape[1] - 1, max_cycles=self.max_cycles
         )
-
         options = {
             "outcome": outcome_encoded,
             "data": data,
@@ -394,14 +368,14 @@ class Solver:
             "timeout": timeout,
         }
 
-        result = exp.explore(self._client, edf, options)
+        exp.explore(options)
 
-        decoded_rules = meta_utils.decode_meta_rules(
-            result.rules, edf, outcome_encoded, encoder
-        )
-        meta_mdf = meta_utils.apply_meta_rules_to_df(decoded_rules, mdf)
+        exp.decode_results(edf, outcome_encoded, encoder)
 
-        self.meta = {"rules": [x.dict() for x in decoded_rules], "df": meta_mdf}
+        rules = exp.result["rules"]
+        meta_mdf = meta_utils.apply_meta_rules_to_df(rules, mdf)
+
+        self.meta = {"rules": rules, "df": meta_mdf}
 
     def cross_validate(self, m_X, m_y, cv=3):
         """
